from .config import parsed_file, commands
import pytest


EXAMPLE_CONFIG = """\
[commands]
upload = bin/python upload.py
test = bin/py.test
run = bin/run
[packages]
foo.a
bar.x"""


@pytest.fixture('function')
def config_file(tmpdir):
    """Create a config file with the given content."""
    def config(content=EXAMPLE_CONFIG):
        data = tmpdir.join('config.ini')
        data.write(content)
        return parsed_file(data.open())
    return config


def test_config__commands__1(config_file):
    """It returns the selected commands from the config file."""
    assert (('bin/py.test', 'bin/python upload.py') ==
            commands(config_file(), ['test', 'upload']))


def test_config__commands__2(config_file):
    """It raises a `RuntimeError` if the commands section is missing."""
    file = config_file("")
    with pytest.raises(RuntimeError) as err:
        commands(file, [])
    assert ('Missing the section [commands] in the config file.' ==
            str(err.value))


def test_config__commands__3(config_file):
    """It raises a `RuntimeError` if a requested command is missing."""
    file = config_file()
    with pytest.raises(RuntimeError) as err:
        commands(file, ['run', 'stop'])
    assert ("Section [commands] in the config file does not contain the key "
            "'stop' you requested to execute." ==
            str(err.value))



from .run import Runner
from .testing import get_packages, test_command
import re
import sys


def test_run____call____1():
    """It returns `True` on success of all packages."""
    runner = Runner([test_command], get_packages('fine', 'finetoo'))
    assert True is runner()


def test_run____call____2(capsys):
    """It prints the tests results to stdout."""
    packages = get_packages('fine', 'finetoo')
    runner = Runner([test_command], packages)
    runner()
    out, err = capsys.readouterr()
    assert 'Running {0} on {1[0]}'.format(test_command, packages) in out
    assert 'Running {0} on {1[1]}'.format(test_command, packages) in out
    assert out.strip().endswith('SUCCESS :-)')


def test_run____call____3(capsys):
    """It stops after the first package with a failure."""
    packages = get_packages('bad', 'fine')
    runner = Runner([test_command], packages)
    runner()
    out, err = capsys.readouterr()
    assert 'Running {0} on {1[0]}'.format(test_command, packages) in out
    assert 'Running {0} on {1[1]}'.format(test_command, packages) not in out
    assert out.strip().endswith('FAILURE :-(')


def test_run____call____4(capsys):
    """It runs multiple commands on each package."""
    packages = get_packages('fine', 'finetoo')
    runner = Runner([test_command, sys.executable], packages)
    runner()
    out, err = capsys.readouterr()
    assert ("""\
Running {0} on {2[0]}
Running {0} on {2[1]}
Running {1} on {2[0]}
Running {1} on {2[1]}""".format(test_command, sys.executable, packages) ==
            "\n".join(re.findall('^Running .* on .*$', out, re.M)))
    assert out.strip().endswith('SUCCESS :-)')



from .compat import ConfigParser


__all__ = ['parsed_file', 'packages', 'commands']


def parsed_file(config_file):
    """Parse an ini-style config file."""
    parser = ConfigParser(allow_no_value=True)
    parser.readfp(config_file)
    return parser


def packages(config):
    """Return the packages defined in the config file as tuple."""
    _assert_section(config, 'packages')
    return tuple(config.options('packages'))


def commands(config, names):
    """Return the list of commands to run."""
    _assert_section(config, 'commands')
    commands = dict(config.items('commands'))
    try:
        return tuple(commands[x] for x in names)
    except KeyError as e:
        raise RuntimeError(
            'Section [commands] in the config file does not contain the '
            'key {.args[0]!r} you requested to execute.'.format(e))


def _assert_section(config, name):
    """Make sure that a section exists in the config file."""
    if name not in config.sections():
        raise RuntimeError(
            'Missing the section [{}] in the config file.'.format(name))



from __future__ import print_function
import os
import subprocess


class Runner:
    """Run the tests."""

    def __init__(self, commands, packages):
        self.commands = commands
        self.packages = packages

    def __call__(self):
        for command in self.commands:
            for package in self.packages:
                if not self.run(command, package):
                    self.render_failure()
                    return False
        self.render_success()
        return True

    def run(self, cmd, package):
        print('Running', cmd, 'on', package)
        cwd = os.getcwd()
        os.chdir(package)
        try:
            process = subprocess.Popen(cmd.split())
            return process.wait() == 0
        finally:
            os.chdir(cwd)

    def render_failure(self):
        print('FAILURE :-(')

    def render_success(self):
        print('SUCCESS :-)')
